@@ -67,6 +67,8 @@ GST_ELEMENT_DETAILS ("Audio sink (ALSA)",
 
 #define DEFAULT_DEVICE		"default"
 #define DEFAULT_DEVICE_NAME	""
+#define SPDIF_PERIOD_SIZE 1536
+#define SPDIF_BUFFER_SIZE 15360
 
 enum
 {
@@ -122,6 +124,18 @@ static GstStaticPadTemplate alsasink_sink_factory =
         "audio/x-raw-int, "
         "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
         "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 24, "
+        "depth = (int) 24, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 32, "
+        "depth = (int) 24, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
+        "audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 16, "
         "depth = (int) 16, "
         "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]; "
@@ -129,7 +143,8 @@ static GstStaticPadTemplate alsasink_sink_factory =
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 8, "
         "depth = (int) 8, "
-        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]")
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ];"
+        "audio/x-iec958")
     );
 
 static void
@@ -201,12 +216,12 @@ gst_alsasink_class_init (GstAlsaSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_DEVICE,
       g_param_spec_string ("device", "Device",
           "ALSA device, as defined in an asound configuration file",
-          DEFAULT_DEVICE, G_PARAM_READWRITE));
+          DEFAULT_DEVICE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
           "Human-readable name of the sound device", DEFAULT_DEVICE_NAME,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -321,10 +336,11 @@ set_hwparams (GstAlsaSink * alsa)
   snd_pcm_hw_params_t *params;
   guint period_time, buffer_time;
 
-  snd_pcm_hw_params_alloca (&params);
+  snd_pcm_hw_params_malloc (&params);
 
-  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %s)",
-      alsa->channels, alsa->rate, snd_pcm_format_name (alsa->format));
+  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %s) "
+      "SPDIF (%d)", alsa->channels, alsa->rate,
+      snd_pcm_format_name (alsa->format), alsa->iec958);
 
   /* start with requested values, if we cannot configure alsa for those values,
    * we set these values to -1, which will leave the default alsa values */
@@ -338,6 +354,16 @@ retry:
   CHECK (snd_pcm_hw_params_set_access (alsa->handle, params, alsa->access),
       wrong_access);
   /* set the sample format */
+  if (alsa->iec958) {
+    /* Try to use big endian first else fallback to le and swap bytes */
+    if (snd_pcm_hw_params_set_format (alsa->handle, params, alsa->format) < 0) {
+      alsa->format = SND_PCM_FORMAT_S16_LE;
+      alsa->need_swap = TRUE;
+      GST_DEBUG_OBJECT (alsa, "falling back to little endian with swapping");
+    } else {
+      alsa->need_swap = FALSE;
+    }
+  }
   CHECK (snd_pcm_hw_params_set_format (alsa->handle, params, alsa->format),
       no_sample_format);
   /* set the count of channels */
@@ -345,7 +371,7 @@ retry:
       no_channels);
   /* set the stream rate */
   rrate = alsa->rate;
-  CHECK (snd_pcm_hw_params_set_rate_near (alsa->handle, params, &rrate, 0),
+  CHECK (snd_pcm_hw_params_set_rate_near (alsa->handle, params, &rrate, NULL),
       no_rate);
   if (rrate != alsa->rate)
     goto rate_match;
@@ -374,7 +400,7 @@ retry:
 
   /* now try to configure the buffer time and period time, if one
    * of those fail, we fall back to the defaults and emit a warning. */
-  if (buffer_time != -1) {
+  if (buffer_time != -1 && !alsa->iec958) {
     /* set the buffer time */
     if ((err = snd_pcm_hw_params_set_buffer_time_near (alsa->handle, params,
                 &buffer_time, &dir)) < 0) {
@@ -385,8 +411,9 @@ retry:
       buffer_time = -1;
       goto retry;
     }
+    GST_DEBUG_OBJECT (alsa, "buffer time %u", buffer_time);
   }
-  if (period_time != -1) {
+  if (period_time != -1 && !alsa->iec958) {
     /* set the period time */
     if ((err = snd_pcm_hw_params_set_period_time_near (alsa->handle, params,
                 &period_time, &dir)) < 0) {
@@ -397,6 +424,18 @@ retry:
       period_time = -1;
       goto retry;
     }
+    GST_DEBUG_OBJECT (alsa, "period time %u", period_time);
+  }
+
+  /* Set buffer size and period size manually for SPDIF */
+  if (G_UNLIKELY (alsa->iec958)) {
+    snd_pcm_uframes_t buffer_size = SPDIF_BUFFER_SIZE;
+    snd_pcm_uframes_t period_size = SPDIF_PERIOD_SIZE;
+
+    CHECK (snd_pcm_hw_params_set_buffer_size_near (alsa->handle, params,
+            &buffer_size), buffer_size);
+    CHECK (snd_pcm_hw_params_set_period_size_near (alsa->handle, params,
+            &period_size, NULL), period_size);
   }
 
   /* write the parameters to device */
@@ -411,6 +450,7 @@ retry:
   GST_DEBUG_OBJECT (alsa, "buffer size %lu, period size %lu", alsa->buffer_size,
       alsa->period_size);
 
+  snd_pcm_hw_params_free (params);
   return 0;
 
   /* ERRORS */
@@ -419,18 +459,21 @@ no_config:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Broken configuration for playback: no configurations available: %s",
             snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 wrong_access:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Access type not available for playback: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_sample_format:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Sample format not available for playback: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_channels:
@@ -448,6 +491,7 @@ no_channels:
           alsa->channels);
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (msg), (snd_strerror (err)));
     g_free (msg);
+    snd_pcm_hw_params_free (params);
     return err;
   }
 no_rate:
@@ -461,24 +505,28 @@ rate_match:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Rate doesn't match (requested %iHz, get %iHz)", alsa->rate, err));
+    snd_pcm_hw_params_free (params);
     return -EINVAL;
   }
 buffer_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get buffer size for playback: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 period_size:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to get period size for playback: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 set_hw_params:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set hw params for playback: %s", snd_strerror (err)));
+    snd_pcm_hw_params_free (params);
     return err;
   }
 }
@@ -489,7 +537,7 @@ set_swparams (GstAlsaSink * alsa)
   int err;
   snd_pcm_sw_params_t *params;
 
-  snd_pcm_sw_params_alloca (&params);
+  snd_pcm_sw_params_malloc (&params);
 
   /* get the current swparams */
   CHECK (snd_pcm_sw_params_current (alsa->handle, params), no_config);
@@ -502,12 +550,18 @@ set_swparams (GstAlsaSink * alsa)
   /* allow the transfer when at least period_size samples can be processed */
   CHECK (snd_pcm_sw_params_set_avail_min (alsa->handle, params,
           alsa->period_size), set_avail);
+
+#if GST_CHECK_ALSA_VERSION(1,0,16)
+  /* snd_pcm_sw_params_set_xfer_align() is deprecated, alignment is always 1 */
+#else
   /* align all transfers to 1 sample */
   CHECK (snd_pcm_sw_params_set_xfer_align (alsa->handle, params, 1), set_align);
+#endif
 
   /* write the parameters to the playback device */
   CHECK (snd_pcm_sw_params (alsa->handle, params), set_sw_params);
 
+  snd_pcm_sw_params_free (params);
   return 0;
 
   /* ERRORS */
@@ -516,6 +570,7 @@ no_config:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to determine current swparams for playback: %s",
             snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
 start_threshold:
@@ -523,24 +578,30 @@ start_threshold:
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set start threshold mode for playback: %s",
             snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
 set_avail:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set avail min for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
+#if !GST_CHECK_ALSA_VERSION(1,0,16)
 set_align:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set transfer align for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
+#endif
 set_sw_params:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
         ("Unable to set sw params for playback: %s", snd_strerror (err)));
+    snd_pcm_sw_params_free (params);
     return err;
   }
 }
@@ -548,6 +609,9 @@ set_sw_params:
 static gboolean
 alsasink_parse_spec (GstAlsaSink * alsa, GstRingBufferSpec * spec)
 {
+  /* Initialize our boolean */
+  alsa->iec958 = FALSE;
+
   switch (spec->type) {
     case GST_BUFTYPE_LINEAR:
       GST_DEBUG_OBJECT (alsa,
@@ -581,6 +645,10 @@ alsasink_parse_spec (GstAlsaSink * alsa, GstRingBufferSpec * spec)
     case GST_BUFTYPE_MU_LAW:
       alsa->format = SND_PCM_FORMAT_MU_LAW;
       break;
+    case GST_BUFTYPE_IEC958:
+      alsa->format = SND_PCM_FORMAT_S16_BE;
+      alsa->iec958 = TRUE;
+      break;
     default:
       goto error;
 
@@ -618,11 +686,14 @@ gst_alsasink_open (GstAudioSink * asink)
 open_error:
   {
     if (err == -EBUSY) {
-      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY, (NULL), ("Device '%s' is busy",
-              alsa->device));
+      GST_ELEMENT_ERROR (alsa, RESOURCE, BUSY,
+          (_("Could not open audio device for playback. "
+                  "Device is being used by another application.")),
+          ("Device '%s' is busy", alsa->device));
     } else {
       GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_WRITE,
-          (NULL), ("Playback open error on device '%s': %s", alsa->device,
+          (_("Could not open audio device for playback.")),
+          ("Playback open error on device '%s': %s", alsa->device,
               snd_strerror (err)));
     }
     return FALSE;
@@ -637,6 +708,14 @@ gst_alsasink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 
   alsa = GST_ALSA_SINK (asink);
 
+  if (spec->format == GST_IEC958) {
+    snd_pcm_close (alsa->handle);
+    alsa->handle = gst_alsa_open_iec958_pcm (GST_OBJECT (alsa));
+    if (G_UNLIKELY (!alsa->handle)) {
+      goto no_iec958;
+    }
+  }
+
   if (!alsasink_parse_spec (alsa, spec))
     goto spec_parse;
 
@@ -649,9 +728,31 @@ gst_alsasink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
   spec->segsize = alsa->period_size * spec->bytes_per_sample;
   spec->segtotal = alsa->buffer_size / alsa->period_size;
 
+  {
+    snd_output_t *out_buf = NULL;
+    char *msg = NULL;
+
+    snd_output_buffer_open (&out_buf);
+    snd_pcm_dump_hw_setup (alsa->handle, out_buf);
+    snd_output_buffer_string (out_buf, &msg);
+    GST_DEBUG_OBJECT (alsa, "Hardware setup: \n%s", msg);
+    snd_output_close (out_buf);
+    snd_output_buffer_open (&out_buf);
+    snd_pcm_dump_sw_setup (alsa->handle, out_buf);
+    snd_output_buffer_string (out_buf, &msg);
+    GST_DEBUG_OBJECT (alsa, "Software setup: \n%s", msg);
+    snd_output_close (out_buf);
+  }
+
   return TRUE;
 
   /* ERRORS */
+no_iec958:
+  {
+    GST_ELEMENT_ERROR (alsa, RESOURCE, OPEN_WRITE, (NULL),
+        ("Could not open IEC958 (SPDIF) device for playback"));
+    return FALSE;
+  }
 spec_parse:
   {
     GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS, (NULL),
@@ -721,8 +822,10 @@ gst_alsasink_close (GstAudioSink * asink)
   GstAlsaSink *alsa = GST_ALSA_SINK (asink);
   gint err;
 
-  CHECK (snd_pcm_close (alsa->handle), close_error);
-  alsa->handle = NULL;
+  if (alsa->handle) {
+    CHECK (snd_pcm_close (alsa->handle), close_error);
+    alsa->handle = NULL;
+  }
   gst_caps_replace (&alsa->cached_caps, NULL);
 
   return TRUE;
@@ -774,18 +877,28 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
   GstAlsaSink *alsa;
   gint err;
   gint cptr;
-  gint16 *ptr;
+  gint16 *ptr = data;
 
   alsa = GST_ALSA_SINK (asink);
 
+  if (alsa->iec958 && alsa->need_swap) {
+    guint i;
+
+    GST_DEBUG_OBJECT (asink, "swapping bytes");
+    for (i = 0; i < length / 2; i++) {
+      ptr[i] = GUINT16_SWAP_LE_BE (ptr[i]);
+    }
+  }
+
+  GST_LOG_OBJECT (asink, "received audio samples buffer of %u bytes", length);
+
   cptr = length / alsa->bytes_per_sample;
-  ptr = data;
 
   GST_ALSA_SINK_LOCK (asink);
   while (cptr > 0) {
     err = snd_pcm_writei (alsa->handle, ptr, cptr);
 
-    GST_DEBUG_OBJECT (asink, "written %d result %d", cptr, err);
+    GST_DEBUG_OBJECT (asink, "written %d frames out of %d", err, cptr);
     if (err < 0) {
       GST_DEBUG_OBJECT (asink, "Write error: %s", snd_strerror (err));
       if (err == -EAGAIN) {
@@ -796,12 +909,12 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
       continue;
     }
 
-    ptr += err * alsa->channels;
+    ptr += snd_pcm_frames_to_bytes (alsa->handle, err);
     cptr -= err;
   }
   GST_ALSA_SINK_UNLOCK (asink);
 
-  return length - cptr;
+  return length - (cptr * alsa->bytes_per_sample);
 
 write_error:
   {
@@ -815,10 +928,21 @@ gst_alsasink_delay (GstAudioSink * asink)
 {
   GstAlsaSink *alsa;
   snd_pcm_sframes_t delay;
+  int res;
 
   alsa = GST_ALSA_SINK (asink);
 
-  snd_pcm_delay (alsa->handle, &delay);
+  res = snd_pcm_delay (alsa->handle, &delay);
+  if (G_UNLIKELY (res < 0)) {
+    /* on errors, report 0 delay */
+    GST_DEBUG_OBJECT (alsa, "snd_pcm_delay returned %d", res);
+    delay = 0;
+  }
+  if (G_UNLIKELY (delay < 0)) {
+    /* make sure we never return a negative delay */
+    GST_WARNING_OBJECT (alsa, "snd_pcm_delay returned negative delay");
+    delay = 0;
+  }
 
   return delay;
 }
