@@ -38,7 +38,7 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_GNOME_VFS_SINK,GstGnomeVFSSinkClass))
 #define GST_IS_GNOME_VFS_SINK(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_GNOME_VFS_SINK))
-#define GST_IS_GNOME_VFS_SINK_CLASS(obj) \
+#define GST_IS_GNOME_VFS_SINK_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_GNOME_VFS_SINK))
 
 typedef struct _GstGnomeVFSSink GstGnomeVFSSink;
@@ -65,7 +65,7 @@ struct _GstGnomeVFSSink
   /* whether we opened the handle ourselves */
   gboolean own_handle;
 
-  guint64  data_written;
+  guint64  current_pos;
 };
 
 struct _GstGnomeVFSSinkClass
