@@ -67,25 +67,13 @@
 GST_DEBUG_CATEGORY_EXTERN (vorbisparse_debug);
 #define GST_CAT_DEFAULT vorbisparse_debug
 
-static GstElementDetails vorbis_parse_details = {
+static const GstElementDetails vorbis_parse_details = {
   "VorbisParse",
   "Codec/Parser/Audio",
   "parse raw vorbis streams",
   "Thomas Vander Stichele <thomas at apestaart dot org>"
 };
 
-/* Filter signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0
-};
-
 static GstStaticPadTemplate vorbis_parse_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -111,6 +99,8 @@ static gboolean vorbis_parse_src_query (GstPad * pad, GstQuery * query);
 static gboolean vorbis_parse_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value);
+static GstFlowReturn vorbis_parse_parse_packet (GstVorbisParse * parse,
+    GstBuffer * buf);
 
 static void
 gst_vorbis_parse_base_init (gpointer g_class)
@@ -130,6 +120,8 @@ gst_vorbis_parse_class_init (GstVorbisParseClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   gstelement_class->change_state = vorbis_parse_change_state;
+
+  klass->parse_packet = GST_DEBUG_FUNCPTR (vorbis_parse_parse_packet);
 }
 
 static void
@@ -137,13 +129,16 @@ gst_vorbis_parse_init (GstVorbisParse * parse, GstVorbisParseClass * g_class)
 {
   parse->sinkpad =
       gst_pad_new_from_static_template (&vorbis_parse_sink_factory, "sink");
-  gst_pad_set_chain_function (parse->sinkpad, vorbis_parse_chain);
-  gst_pad_set_event_function (parse->sinkpad, vorbis_parse_sink_event);
+  gst_pad_set_chain_function (parse->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_chain));
+  gst_pad_set_event_function (parse->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_sink_event));
   gst_element_add_pad (GST_ELEMENT (parse), parse->sinkpad);
 
   parse->srcpad =
       gst_pad_new_from_static_template (&vorbis_parse_src_factory, "src");
-  gst_pad_set_query_function (parse->srcpad, vorbis_parse_src_query);
+  gst_pad_set_query_function (parse->srcpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_src_query));
   gst_element_add_pad (GST_ELEMENT (parse), parse->srcpad);
 }
 
@@ -191,12 +186,23 @@ vorbis_parse_set_header_on_caps (GstVorbisParse * parse, GstCaps * caps)
   g_value_unset (&array);
 }
 
+static void
+vorbis_parse_drain_event_queue (GstVorbisParse * parse)
+{
+  while (parse->event_queue->length) {
+    GstEvent *event;
+
+    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
+    gst_pad_event_default (parse->sinkpad, event);
+  }
+}
+
 static void
 vorbis_parse_push_headers (GstVorbisParse * parse)
 {
   /* mark and put on caps */
   GstCaps *caps;
-  GstBuffer *outbuf;
+  GstBuffer *outbuf, *outbuf1, *outbuf2, *outbuf3;
   ogg_packet packet;
 
   /* get the headers into the caps, passing them to vorbis as we go */
@@ -206,18 +212,16 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   gst_pad_set_caps (parse->srcpad, caps);
   gst_caps_unref (caps);
 
-  /* push out buffers, ignoring return value... */
   outbuf = GST_BUFFER_CAST (parse->streamheader->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
   packet.bytes = GST_BUFFER_SIZE (outbuf);
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 1;
   packet.e_o_s = 0;
+  packet.b_o_s = 1;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
   parse->sample_rate = parse->vi.rate;
-
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf);
+  outbuf1 = outbuf;
 
   outbuf = GST_BUFFER_CAST (parse->streamheader->next->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
@@ -225,10 +229,9 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 2;
   packet.e_o_s = 0;
+  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
-
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf);
+  outbuf2 = outbuf;
 
   outbuf = GST_BUFFER_CAST (parse->streamheader->next->next->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
@@ -236,10 +239,20 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 3;
   packet.e_o_s = 0;
+  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
+  outbuf3 = outbuf;
 
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf);
+  /* first process queued events */
+  vorbis_parse_drain_event_queue (parse);
+
+  /* push out buffers, ignoring return value... */
+  gst_buffer_set_caps (outbuf1, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf1);
+  gst_buffer_set_caps (outbuf2, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf2);
+  gst_buffer_set_caps (outbuf3, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf3);
 
   g_list_free (parse->streamheader);
   parse->streamheader = NULL;
@@ -256,6 +269,12 @@ vorbis_parse_clear_queue (GstVorbisParse * parse)
     buf = GST_BUFFER_CAST (g_queue_pop_head (parse->buffer_queue));
     gst_buffer_unref (buf);
   }
+  while (parse->event_queue->length) {
+    GstEvent *event;
+
+    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
+    gst_event_unref (event);
+  }
 }
 
 static GstFlowReturn
@@ -289,6 +308,10 @@ vorbis_parse_drain_queue_prematurely (GstVorbisParse * parse)
    * didn't-get-a-granulepos-on-the-last-packet case. Assuming a continuous
    * stream. */
 
+  /* if we got EOS before any buffers came, go ahead and push the other events
+   * first */
+  vorbis_parse_drain_event_queue (parse);
+
   while (!g_queue_is_empty (parse->buffer_queue)) {
     GstBuffer *buf;
 
@@ -376,15 +399,10 @@ vorbis_parse_queue_buffer (GstVorbisParse * parse, GstBuffer * buf)
 }
 
 static GstFlowReturn
-vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
+vorbis_parse_parse_packet (GstVorbisParse * parse, GstBuffer * buf)
 {
   GstFlowReturn ret;
-  GstBuffer *buf;
-  GstVorbisParse *parse;
 
-  parse = GST_VORBIS_PARSE (gst_pad_get_parent (pad));
-
-  buf = GST_BUFFER (buffer);
   parse->packetno++;
 
   if (parse->packetno <= 3) {
@@ -399,7 +417,29 @@ vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
     ret = vorbis_parse_queue_buffer (parse, buf);
   }
 
-  gst_object_unref (parse);
+  return ret;
+}
+
+static GstFlowReturn
+vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
+{
+  GstVorbisParseClass *klass;
+  GstVorbisParse *parse;
+
+  parse = GST_VORBIS_PARSE (GST_PAD_PARENT (pad));
+  klass = GST_VORBIS_PARSE_CLASS (G_OBJECT_GET_CLASS (parse));
+
+  g_assert (klass->parse_packet != NULL);
+
+  return klass->parse_packet (parse, buffer);
+}
+
+static gboolean
+vorbis_parse_queue_event (GstVorbisParse * parse, GstEvent * event)
+{
+  GstFlowReturn ret = TRUE;
+
+  g_queue_push_tail (parse->event_queue, event);
 
   return ret;
 }
@@ -424,7 +464,10 @@ vorbis_parse_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
     default:
-      ret = gst_pad_event_default (pad, event);
+      if (!parse->streamheader_sent && GST_EVENT_IS_SERIALIZED (event))
+        ret = vorbis_parse_queue_event (parse, event);
+      else
+        ret = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -537,8 +580,8 @@ vorbis_parse_src_query (GstPad * pad, GstQuery * query)
 
       gst_query_set_position (query, format, value);
 
-      GST_LOG_OBJECT (parse,
-          "query %u: peer returned granulepos: %llu - we return %llu (format %u)",
+      GST_LOG_OBJECT (parse, "query %p: peer returned granulepos: %"
+          G_GUINT64_FORMAT " - we return %" G_GUINT64_FORMAT " (format %u)",
           query, granulepos, value, format);
 
       break;
@@ -597,6 +640,7 @@ vorbis_parse_change_state (GstElement * element, GstStateChange transition)
       parse->packetno = 0;
       parse->streamheader_sent = FALSE;
       parse->buffer_queue = g_queue_new ();
+      parse->event_queue = g_queue_new ();
       break;
     default:
       break;
@@ -611,6 +655,8 @@ vorbis_parse_change_state (GstElement * element, GstStateChange transition)
       vorbis_parse_clear_queue (parse);
       g_queue_free (parse->buffer_queue);
       parse->buffer_queue = NULL;
+      g_queue_free (parse->event_queue);
+      parse->event_queue = NULL;
       break;
     default:
       break;
