@@ -20,7 +20,7 @@
 /**
  * SECTION:gstringbuffer
  * @short_description: Base class for audio ringbuffer implementations
- * @see_also: gstbaseaudiosink
+ * @see_also: #GstBaseAudioSink, #GstAudioSink
  *
  * <refsect2>
  * <para>
@@ -134,8 +134,8 @@ gst_ring_buffer_finalize (GObject * object)
 
 typedef struct
 {
-  GstBufferFormat format;
-  guint8 silence[4];
+  const GstBufferFormat format;
+  const guint8 silence[4];
 } FormatDef;
 
 static const FormatDef linear_defs[4 * 2 * 2] = {
@@ -189,7 +189,7 @@ build_linear_format (int depth, int width, int unsignd, int big_endian)
         formats = &linear24_defs[8];
         break;
       default:
-        return GST_UNKNOWN;
+        return NULL;
     }
   } else {
     switch (depth) {
@@ -206,7 +206,7 @@ build_linear_format (int depth, int width, int unsignd, int big_endian)
         formats = &linear_defs[12];
         break;
       default:
-        return GST_UNKNOWN;
+        return NULL;
     }
   }
   if (unsignd)
@@ -258,6 +258,7 @@ gst_ring_buffer_debug_spec_buff (GstRingBufferSpec * spec)
   GST_DEBUG ("acquire ringbuffer: latency time: %" G_GINT64_FORMAT " usec",
       spec->latency_time);
   GST_DEBUG ("acquire ringbuffer: total segments: %d", spec->segtotal);
+  GST_DEBUG ("acquire ringbuffer: latency segments: %d", spec->seglatency);
   GST_DEBUG ("acquire ringbuffer: segment size: %d bytes = %d samples",
       spec->segsize, spec->segsize / spec->bytes_per_sample);
   GST_DEBUG ("acquire ringbuffer: buffer size: %d bytes = %d samples",
@@ -286,11 +287,6 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
   /* we have to differentiate between int and float formats */
   mimetype = gst_structure_get_name (structure);
 
-  /* get rate and channels */
-  if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
-          gst_structure_get_int (structure, "channels", &spec->channels)))
-    goto parse_error;
-
   if (!strncmp (mimetype, "audio/x-raw-int", 15)) {
     gint endianness;
     const FormatDef *def;
@@ -299,7 +295,9 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     spec->type = GST_BUFTYPE_LINEAR;
 
     /* extract the needed information from the cap */
-    if (!(gst_structure_get_int (structure, "width", &spec->width) &&
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels) &&
+            gst_structure_get_int (structure, "width", &spec->width) &&
             gst_structure_get_int (structure, "depth", &spec->depth) &&
             gst_structure_get_boolean (structure, "signed", &spec->sign)))
       goto parse_error;
@@ -317,6 +315,9 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     def = build_linear_format (spec->depth, spec->width, spec->sign ? 0 : 1,
         spec->bigend ? 1 : 0);
 
+    if (def == NULL)
+      goto parse_error;
+
     spec->format = def->format;
 
     bytes = spec->width >> 3;
@@ -330,8 +331,10 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
 
     spec->type = GST_BUFTYPE_FLOAT;
 
-    /* get layout */
-    if (!gst_structure_get_int (structure, "width", &spec->width))
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels) &&
+            gst_structure_get_int (structure, "width", &spec->width)))
       goto parse_error;
 
     /* match layout to format wrt to endianness */
@@ -350,6 +353,11 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     /* float silence is all zeros.. */
     memset (spec->silence_sample, 0, 32);
   } else if (!strncmp (mimetype, "audio/x-alaw", 12)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels)))
+      goto parse_error;
+
     spec->type = GST_BUFTYPE_A_LAW;
     spec->format = GST_A_LAW;
     spec->width = 8;
@@ -357,12 +365,37 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     for (i = 0; i < spec->channels; i++)
       spec->silence_sample[i] = 0xd5;
   } else if (!strncmp (mimetype, "audio/x-mulaw", 13)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels)))
+      goto parse_error;
+
     spec->type = GST_BUFTYPE_MU_LAW;
     spec->format = GST_MU_LAW;
     spec->width = 8;
     spec->depth = 8;
     for (i = 0; i < spec->channels; i++)
       spec->silence_sample[i] = 0xff;
+  } else if (!strncmp (mimetype, "audio/x-iec958", 14)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate)))
+      goto parse_error;
+
+    spec->type = GST_BUFTYPE_IEC958;
+    spec->format = GST_IEC958;
+    spec->width = 16;
+    spec->depth = 16;
+    spec->channels = 2;
+  } else if (!strncmp (mimetype, "audio/x-ac3", 11)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate)))
+      goto parse_error;
+
+    spec->type = GST_BUFTYPE_AC3;
+    spec->format = GST_AC3;
+    spec->width = 16;
+    spec->depth = 16;
+    spec->channels = 2;
   } else {
     goto parse_error;
   }
@@ -382,6 +415,9 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
   spec->segsize -= spec->segsize % spec->bytes_per_sample;
 
   spec->segtotal = spec->buffer_time / spec->latency_time;
+  /* leave the latency undefined now, implementations can change it but if it's
+   * not changed, we assume the same value as segtotal */
+  spec->seglatency = -1;
 
   gst_ring_buffer_debug_spec_caps (spec);
   gst_ring_buffer_debug_spec_buff (spec);
@@ -617,6 +653,11 @@ gst_ring_buffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   if (G_UNLIKELY ((bps = buf->spec.bytes_per_sample) == 0))
     goto invalid_bps;
 
+  /* if the seglatency was overwritten with something else than -1, use it, else
+   * assume segtotal as the latency */
+  if (buf->spec.seglatency == -1)
+    buf->spec.seglatency = buf->spec.segtotal;
+
   segsize = buf->spec.segsize;
 
   buf->samples_per_seg = segsize / bps;
@@ -808,6 +849,9 @@ gst_ring_buffer_start (GstRingBuffer * buf)
   if (G_UNLIKELY (buf->abidata.ABI.flushing))
     goto flushing;
 
+  if (G_UNLIKELY (!buf->acquired))
+    goto not_acquired;
+
   /* if stopped, set to started */
   res = g_atomic_int_compare_and_exchange (&buf->state,
       GST_RING_BUFFER_STATE_STOPPED, GST_RING_BUFFER_STATE_STARTED);
@@ -849,6 +893,13 @@ done:
 
 flushing:
   {
+    GST_DEBUG_OBJECT (buf, "we are flushing");
+    GST_OBJECT_UNLOCK (buf);
+    return FALSE;
+  }
+not_acquired:
+  {
+    GST_DEBUG_OBJECT (buf, "we are not acquired");
     GST_OBJECT_UNLOCK (buf);
     return FALSE;
   }
@@ -915,13 +966,24 @@ gst_ring_buffer_pause (GstRingBuffer * buf)
   if (G_UNLIKELY (buf->abidata.ABI.flushing))
     goto flushing;
 
+  if (G_UNLIKELY (!buf->acquired))
+    goto not_acquired;
+
   res = gst_ring_buffer_pause_unlocked (buf);
   GST_OBJECT_UNLOCK (buf);
 
   return res;
 
+  /* ERRORS */
 flushing:
   {
+    GST_DEBUG_OBJECT (buf, "we are flushing");
+    GST_OBJECT_UNLOCK (buf);
+    return FALSE;
+  }
+not_acquired:
+  {
+    GST_DEBUG_OBJECT (buf, "not acquired");
     GST_OBJECT_UNLOCK (buf);
     return FALSE;
   }
@@ -989,6 +1051,12 @@ done:
  * implementation uses another internal buffer between the audio
  * device.
  *
+ * For playback ringbuffers this is the amount of samples transfered from the
+ * ringbuffer to the device but still not played.
+ *
+ * For capture ringbuffers this is the amount of samples in the device that are
+ * not yet transfered to the ringbuffer.
+ *
  * Returns: The number of samples queued in the audio device.
  *
  * MT safe.
@@ -1001,18 +1069,23 @@ gst_ring_buffer_delay (GstRingBuffer * buf)
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), 0);
 
-  res = 0;
-
   /* buffer must be acquired */
   if (G_UNLIKELY (!gst_ring_buffer_is_acquired (buf)))
-    goto done;
+    goto not_acquired;
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
   if (G_LIKELY (rclass->delay))
     res = rclass->delay (buf);
+  else
+    res = 0;
 
-done:
   return res;
+
+not_acquired:
+  {
+    GST_DEBUG_OBJECT (buf, "not acquired");
+    return 0;
+  }
 }
 
 /**
@@ -1020,7 +1093,8 @@ done:
  * @buf: the #GstRingBuffer to query
  *
  * Get the number of samples that were processed by the ringbuffer
- * since it was last started.
+ * since it was last started. This does not include the number of samples not
+ * yet processed (see gst_ring_buffer_delay()).
  *
  * Returns: The number of samples processed by the ringbuffer.
  *
@@ -1030,26 +1104,15 @@ guint64
 gst_ring_buffer_samples_done (GstRingBuffer * buf)
 {
   gint segdone;
-  guint64 raw, samples;
-  guint delay;
+  guint64 samples;
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), 0);
 
   /* get the amount of segments we processed */
   segdone = g_atomic_int_get (&buf->segdone);
 
-  /* and the number of samples not yet processed */
-  delay = gst_ring_buffer_delay (buf);
-
-  raw = samples = ((guint64) segdone) * buf->samples_per_seg;
-
-  if (G_LIKELY (samples >= delay))
-    samples -= delay;
-  else
-    samples = 0;
-
-  GST_DEBUG_OBJECT (buf, "processed samples: raw %llu, delay %u, real %llu",
-      raw, delay, samples);
+  /* convert to samples */
+  samples = ((guint64) segdone) * buf->samples_per_seg;
 
   return samples;
 }
@@ -1177,33 +1240,142 @@ no_start:
   }
 }
 
+#define FWD_SAMPLES(s,se,d,de)		 	\
+G_STMT_START {					\
+  /* no rate conversion */			\
+  guint towrite = MIN (se + bps - s, de - d);	\
+  /* simple copy */				\
+  if (!skip)					\
+    memcpy (d, s, towrite);			\
+  in_samples -= towrite / bps;			\
+  out_samples -= towrite / bps;			\
+  s += towrite;					\
+  GST_DEBUG ("copy %u bytes", towrite);		\
+} G_STMT_END
+
+/* in_samples >= out_samples, rate > 1.0 */
+#define FWD_UP_SAMPLES(s,se,d,de) 	 	\
+G_STMT_START {					\
+  guint8 *sb = s, *db = d;			\
+  while (s <= se && d < de) {			\
+    if (!skip)					\
+      memcpy (d, s, bps);			\
+    s += bps;					\
+    *accum += outr;				\
+    if ((*accum << 1) >= inr) {			\
+      *accum -= inr;				\
+      d += bps;					\
+    }						\
+  }						\
+  in_samples -= (s - sb)/bps;			\
+  out_samples -= (d - db)/bps;			\
+  GST_DEBUG ("fwd_up end %d/%d",*accum,*toprocess);	\
+} G_STMT_END
+
+/* out_samples > in_samples, for rates smaller than 1.0 */
+#define FWD_DOWN_SAMPLES(s,se,d,de) 	 	\
+G_STMT_START {					\
+  guint8 *sb = s, *db = d;			\
+  while (s <= se && d < de) {			\
+    if (!skip)					\
+      memcpy (d, s, bps);			\
+    d += bps;					\
+    *accum += inr;				\
+    if ((*accum << 1) >= outr) {		\
+      *accum -= outr;				\
+      s += bps;					\
+    }						\
+  }						\
+  in_samples -= (s - sb)/bps;			\
+  out_samples -= (d - db)/bps;			\
+  GST_DEBUG ("fwd_down end %d/%d",*accum,*toprocess);	\
+} G_STMT_END
+
+#define REV_UP_SAMPLES(s,se,d,de) 	 	\
+G_STMT_START {					\
+  guint8 *sb = se, *db = d;			\
+  while (s <= se && d < de) {			\
+    if (!skip)					\
+      memcpy (d, se, bps);			\
+    se -= bps;					\
+    *accum += outr;				\
+    while ((*accum << 1) >= inr) {		\
+      *accum -= inr;				\
+      d += bps;					\
+    }						\
+  }						\
+  in_samples -= (sb - se)/bps;			\
+  out_samples -= (d - db)/bps;			\
+  GST_DEBUG ("rev_up end %d/%d",*accum,*toprocess);	\
+} G_STMT_END
+
+#define REV_DOWN_SAMPLES(s,se,d,de) 	 	\
+G_STMT_START {					\
+  guint8 *sb = se, *db = d;			\
+  while (s <= se && d < de) {			\
+    if (!skip)					\
+      memcpy (d, se, bps);			\
+    d += bps;					\
+    *accum += inr;				\
+    while ((*accum << 1) >= outr) {		\
+      *accum -= outr;				\
+      se -= bps;				\
+    }						\
+  }						\
+  in_samples -= (sb - se)/bps;			\
+  out_samples -= (d - db)/bps;			\
+  GST_DEBUG ("rev_down end %d/%d",*accum,*toprocess);	\
+} G_STMT_END
+
 /**
- * gst_ring_buffer_commit:
+ * gst_ring_buffer_commit_full:
  * @buf: the #GstRingBuffer to commit
  * @sample: the sample position of the data
  * @data: the data to commit
- * @len: the number of samples in the data to commit
+ * @in_samples: the number of samples in the data to commit
+ * @out_samples: the number of samples to write to the ringbuffer
+ * @accum: accumulator for rate conversion.
  *
- * Commit @len samples pointed to by @data to the ringbuffer
- * @buf. The first sample should be written at position @sample in
- * the ringbuffer.
+ * Commit @in_samples samples pointed to by @data to the ringbuffer @buf. 
  *
- * @len does not need to be a multiple of the segment size of the ringbuffer
- * although it is recommended for optimal performance.
+ * @in_samples and @out_samples define the rate conversion to perform on the the
+ * samples in @data. For negative rates, @out_samples must be negative and
+ * @in_samples positive.
  *
- * Returns: The number of samples written to the ringbuffer or -1 on
- * error.
+ * When @out_samples is positive, the first sample will be written at position @sample
+ * in the ringbuffer. When @out_samples is negative, the last sample will be written to
+ * @sample in reverse order.
+ *
+ * @out_samples does not need to be a multiple of the segment size of the ringbuffer
+ * although it is recommended for optimal performance. 
+ *
+ * @accum will hold a temporary accumulator used in rate conversion and should be
+ * set to 0 when this function is first called. In case the commit operation is
+ * interrupted, one can resume the processing by passing the previously returned
+ * @accum value back to this function.
+ *
+ * Returns: The number of samples written to the ringbuffer or -1 on error. The
+ * number of samples written can be less than @out_samples when @buf was interrupted
+ * with a flush or stop.
+ *
+ * Since: 0.10.11.
  *
  * MT safe.
  */
 guint
-gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
-    guint len)
+gst_ring_buffer_commit_full (GstRingBuffer * buf, guint64 * sample,
+    guchar * data, gint in_samples, gint out_samples, gint * accum)
 {
   gint segdone;
   gint segsize, segtotal, bps, sps;
-  guint8 *dest;
-  guint to_write;
+  guint8 *dest, *data_end;
+  gint writeseg, sampleoff;
+  gint *toprocess;
+  gint inr, outr;
+  gboolean reverse;
+
+  if (G_UNLIKELY (in_samples == 0 || out_samples == 0))
+    return in_samples;
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), -1);
   g_return_val_if_fail (buf->data != NULL, -1);
@@ -1215,16 +1387,32 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
   bps = buf->spec.bytes_per_sample;
   sps = buf->samples_per_seg;
 
-  to_write = len;
-  /* write out all samples */
-  while (to_write > 0) {
-    gint sampleslen;
-    gint writeseg, sampleoff;
+  reverse = out_samples < 0;
+  out_samples = ABS (out_samples);
 
-    /* figure out the segment and the offset inside the segment where
-     * the sample should be written. */
-    writeseg = sample / sps;
-    sampleoff = (sample % sps);
+  if (in_samples >= out_samples)
+    toprocess = &in_samples;
+  else
+    toprocess = &out_samples;
+
+  inr = in_samples - 1;
+  outr = out_samples - 1;
+
+  /* data_end points to the last sample we have to write, not past it. This is
+   * needed to properly handle reverse playback: it points to the last sample. */
+  data_end = data + (bps * inr);
+
+  /* figure out the segment and the offset inside the segment where
+   * the first sample should be written. */
+  writeseg = *sample / sps;
+  sampleoff = (*sample % sps) * bps;
+
+  /* write out all samples */
+  while (*toprocess > 0) {
+    gint avail;
+    guint8 *d, *d_end;
+    gint ws;
+    gboolean skip;
 
     while (TRUE) {
       gint diff;
@@ -1236,21 +1424,23 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
       diff = writeseg - segdone;
 
       GST_DEBUG
-          ("pointer at %d, sample %llu, write to %d-%d, to_write %d, diff %d, segtotal %d, segsize %d",
-          segdone, sample, writeseg, sampleoff, to_write, diff, segtotal, sps);
+          ("pointer at %d, write to %d-%d, diff %d, segtotal %d, segsize %d",
+          segdone, writeseg, sampleoff, diff, segtotal, segsize);
 
-      /* segment too far ahead, we need to drop, hopefully UNLIKELY */
+      /* segment too far ahead, writer too slow, we need to drop, hopefully UNLIKELY */
       if (G_UNLIKELY (diff < 0)) {
         /* we need to drop one segment at a time, pretend we wrote a
          * segment. */
-        sampleslen = MIN (sps, to_write);
-        goto next;
+        skip = TRUE;
+        break;
       }
 
       /* write segment is within writable range, we can break the loop and
        * start writing the data. */
-      if (diff < segtotal)
+      if (diff < segtotal) {
+        skip = FALSE;
         break;
+      }
 
       /* else we need to wait for the segment to become writable. */
       if (!wait_segment (buf))
@@ -1258,31 +1448,80 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
     }
 
     /* we can write now */
-    writeseg = writeseg % segtotal;
-    sampleslen = MIN (sps - sampleoff, to_write);
+    ws = writeseg % segtotal;
+    avail = MIN (segsize - sampleoff, bps * out_samples);
 
-    GST_DEBUG_OBJECT (buf, "write @%p seg %d, off %d, sampleslen %d",
-        dest + writeseg * segsize, writeseg, sampleoff, sampleslen);
+    d = dest + (ws * segsize) + sampleoff;
+    d_end = d + avail;
+    *sample += avail / bps;
 
-    memcpy (dest + (writeseg * segsize) + (sampleoff * bps), data,
-        (sampleslen * bps));
+    GST_DEBUG_OBJECT (buf, "write @%p seg %d, sps %d, off %d, avail %d",
+        dest + ws * segsize, ws, sps, sampleoff, avail);
 
-  next:
-    to_write -= sampleslen;
-    sample += sampleslen;
-    data += sampleslen * bps;
+    if (G_LIKELY (inr == outr && !reverse)) {
+      /* no rate conversion, simply copy samples */
+      FWD_SAMPLES (data, data_end, d, d_end);
+    } else if (!reverse) {
+      if (inr >= outr)
+        /* forward speed up */
+        FWD_UP_SAMPLES (data, data_end, d, d_end);
+      else
+        /* forward slow down */
+        FWD_DOWN_SAMPLES (data, data_end, d, d_end);
+    } else {
+      if (inr >= outr)
+        /* reverse speed up */
+        REV_UP_SAMPLES (data, data_end, d, d_end);
+      else
+        /* reverse slow down */
+        REV_DOWN_SAMPLES (data, data_end, d, d_end);
+    }
+
+    /* for the next iteration we write to the next segment at the beginning. */
+    writeseg++;
+    sampleoff = 0;
   }
+  /* we consumed all samples here */
+  data = data_end + bps;
 
-  return len - to_write;
+done:
+  return inr - ((data_end - data) / bps);
 
   /* ERRORS */
 not_started:
   {
     GST_DEBUG_OBJECT (buf, "stopped processing");
-    return len - to_write;
+    goto done;
   }
 }
 
+/**
+ * gst_ring_buffer_commit:
+ * @buf: the #GstRingBuffer to commit
+ * @sample: the sample position of the data
+ * @data: the data to commit
+ * @len: the number of samples in the data to commit
+ *
+ * Same as gst_ring_buffer_commit_full() but with a in_samples and out_samples
+ * equal to @len, ignoring accum.
+ *
+ * Returns: The number of samples written to the ringbuffer or -1 on
+ * error.
+ *
+ * MT safe.
+ */
+guint
+gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
+    guint len)
+{
+  guint res;
+  guint64 samplep = sample;
+
+  res = gst_ring_buffer_commit_full (buf, &samplep, data, len, len, NULL);
+
+  return res;
+}
+
 /**
  * gst_ring_buffer_read:
  * @buf: the #GstRingBuffer to read from
@@ -1310,6 +1549,7 @@ gst_ring_buffer_read (GstRingBuffer * buf, guint64 sample, guchar * data,
   gint segdone;
   gint segsize, segtotal, bps, sps;
   guint8 *dest;
+  guint to_read;
 
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), -1);
   g_return_val_if_fail (buf->data != NULL, -1);
@@ -1321,13 +1561,14 @@ gst_ring_buffer_read (GstRingBuffer * buf, guint64 sample, guchar * data,
   bps = buf->spec.bytes_per_sample;
   sps = buf->samples_per_seg;
 
+  to_read = len;
   /* read enough samples */
-  while (len > 0) {
+  while (to_read > 0) {
     gint sampleslen;
     gint readseg, sampleoff;
 
     /* figure out the segment and the offset inside the segment where
-     * the sample should be written. */
+     * the sample should be read from. */
     readseg = sample / sps;
     sampleoff = (sample % sps);
 
@@ -1337,33 +1578,29 @@ gst_ring_buffer_read (GstRingBuffer * buf, guint64 sample, guchar * data,
       /* get the currently processed segment */
       segdone = g_atomic_int_get (&buf->segdone) - buf->segbase;
 
-      /* see how far away it is from the read segment */
+      /* see how far away it is from the read segment, normally segdone (where
+       * the hardware is writing) is bigger than readseg (where software is
+       * reading) */
       diff = segdone - readseg;
 
       GST_DEBUG
-          ("pointer at %d, sample %llu, read from %d-%d, len %d, diff %d, segtotal %d, segsize %d",
-          segdone, sample, readseg, sampleoff, len, diff, segtotal, segsize);
+          ("pointer at %d, sample %llu, read from %d-%d, to_read %d, diff %d, segtotal %d, segsize %d",
+          segdone, sample, readseg, sampleoff, to_read, diff, segtotal,
+          segsize);
 
-      /* segment too far ahead, we need to drop */
-      if (diff < 0) {
-        /* we need to drop one segment at a time, pretend we read an
-         * empty segment. */
-        sampleslen = MIN (sps, len);
+      /* segment too far ahead, reader too slow */
+      if (G_UNLIKELY (diff >= segtotal)) {
+        /* pretend we read an empty segment. */
+        sampleslen = MIN (sps, to_read);
         memcpy (data, buf->empty_seg, sampleslen * bps);
         goto next;
       }
 
       /* read segment is within readable range, we can break the loop and
        * start reading the data. */
-      if (diff > 0 && diff < segtotal)
+      if (diff > 0)
         break;
 
-      /* flush if diff has grown bigger than ringbuffer */
-      if (diff >= segtotal) {
-        gst_ring_buffer_clear_all (buf);
-        buf->segdone = readseg;
-      }
-
       /* else we need to wait for the segment to become readable. */
       if (!wait_segment (buf))
         goto not_started;
@@ -1371,27 +1608,27 @@ gst_ring_buffer_read (GstRingBuffer * buf, guint64 sample, guchar * data,
 
     /* we can read now */
     readseg = readseg % segtotal;
-    sampleslen = MIN (sps - sampleoff, len);
+    sampleslen = MIN (sps - sampleoff, to_read);
 
-    GST_DEBUG_OBJECT (buf, "read @%p seg %d, off %d, len %d",
+    GST_DEBUG_OBJECT (buf, "read @%p seg %d, off %d, sampleslen %d",
         dest + readseg * segsize, readseg, sampleoff, sampleslen);
 
     memcpy (data, dest + (readseg * segsize) + (sampleoff * bps),
         (sampleslen * bps));
 
   next:
-    len -= sampleslen;
+    to_read -= sampleslen;
     sample += sampleslen;
     data += sampleslen * bps;
   }
 
-  return len;
+  return len - to_read;
 
   /* ERRORS */
 not_started:
   {
     GST_DEBUG_OBJECT (buf, "stopped processing");
-    return -1;
+    return len - to_read;
   }
 }
 
@@ -1529,5 +1766,5 @@ gst_ring_buffer_may_start (GstRingBuffer * buf, gboolean allowed)
   g_return_if_fail (GST_IS_RING_BUFFER (buf));
 
   GST_LOG_OBJECT (buf, "may start: %d", allowed);
-  gst_atomic_int_set (&buf->abidata.ABI.may_start, allowed);
+  g_atomic_int_set (&buf->abidata.ABI.may_start, allowed);
 }
