@@ -24,8 +24,10 @@
  * SECTION:element-adder
  *
  * <refsect2>
+ * <para>
  * The Adder allows to mix several streams into one by adding the data.
  * Mixed data is clamped to the min/max values of the data format.
+ * </para>
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
@@ -65,42 +67,71 @@
 #define MIN_UINT_16 ((guint16)(0x0000))
 #define MIN_UINT_8  ((guint8) (0x00))
 
-GST_DEBUG_CATEGORY_STATIC (gst_adder_debug);
 #define GST_CAT_DEFAULT gst_adder_debug
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 /* elementfactory information */
-static const GstElementDetails adder_details = GST_ELEMENT_DETAILS ("Adder",
-    "Generic/Audio",
-    "Add N audio channels together",
-    "Thomas <thomas@apestaart.org>");
+
+#define CAPS \
+  "audio/x-raw-int, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) BYTE_ORDER, " \
+  "width = (int) 32, " \
+  "depth = (int) 32, " \
+  "signed = (boolean) { true, false } ;" \
+  "audio/x-raw-int, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) BYTE_ORDER, " \
+  "width = (int) 16, " \
+  "depth = (int) 16, " \
+  "signed = (boolean) { true, false } ;" \
+  "audio/x-raw-int, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) BYTE_ORDER, " \
+  "width = (int) 8, " \
+  "depth = (int) 8, " \
+  "signed = (boolean) { true, false } ;" \
+  "audio/x-raw-float, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) BYTE_ORDER, " \
+  "width = (int) { 32, 64 }"
 
 static GstStaticPadTemplate gst_adder_src_template =
-    GST_STATIC_PAD_TEMPLATE ("src",
+GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
-        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS (CAPS)
     );
 
 static GstStaticPadTemplate gst_adder_sink_template =
-    GST_STATIC_PAD_TEMPLATE ("sink%d",
+GST_STATIC_PAD_TEMPLATE ("sink%d",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
-        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS (CAPS)
     );
 
 static void gst_adder_class_init (GstAdderClass * klass);
+
 static void gst_adder_init (GstAdder * adder);
+
 static void gst_adder_finalize (GObject * object);
 
 static gboolean gst_adder_setcaps (GstPad * pad, GstCaps * caps);
+
 static gboolean gst_adder_query (GstPad * pad, GstQuery * query);
+
 static gboolean gst_adder_src_event (GstPad * pad, GstEvent * event);
+
 static gboolean gst_adder_sink_event (GstPad * pad, GstEvent * event);
 
 static GstPad *gst_adder_request_new_pad (GstElement * element,
     GstPadTemplate * temp, const gchar * unused);
+static void gst_adder_release_pad (GstElement * element, GstPad * pad);
+
 static GstStateChangeReturn gst_adder_change_state (GstElement * element,
     GstStateChange transition);
 
@@ -124,12 +155,13 @@ gst_adder_get_type (void)
 
     adder_type = g_type_register_static (GST_TYPE_ELEMENT, "GstAdder",
         &adder_info, 0);
-    GST_DEBUG_CATEGORY_INIT (gst_adder_debug, "adder", 0,
+    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "adder", 0,
         "audio channel mixing element");
   }
   return adder_type;
 }
 
+/* clipping versions */
 #define MAKE_FUNC(name,type,ttype,min,max)                      \
 static void name (type *out, type *in, gint bytes) {            \
   gint i;                                                       \
@@ -137,6 +169,14 @@ static void name (type *out, type *in, gint bytes) {            \
     out[i] = CLAMP ((ttype)out[i] + (ttype)in[i], min, max);    \
 }
 
+/* non-clipping versions (for float) */
+#define MAKE_FUNC_NC(name,type,ttype)                           \
+static void name (type *out, type *in, gint bytes) {            \
+  gint i;                                                       \
+  for (i = 0; i < bytes / sizeof (type); i++)                   \
+    out[i] = (ttype)out[i] + (ttype)in[i];                      \
+}
+
 /* *INDENT-OFF* */
 MAKE_FUNC (add_int32, gint32, gint64, MIN_INT_32, MAX_INT_32)
 MAKE_FUNC (add_int16, gint16, gint32, MIN_INT_16, MAX_INT_16)
@@ -144,20 +184,62 @@ MAKE_FUNC (add_int8, gint8, gint16, MIN_INT_8, MAX_INT_8)
 MAKE_FUNC (add_uint32, guint32, guint64, MIN_UINT_32, MAX_UINT_32)
 MAKE_FUNC (add_uint16, guint16, guint32, MIN_UINT_16, MAX_UINT_16)
 MAKE_FUNC (add_uint8, guint8, guint16, MIN_UINT_8, MAX_UINT_8)
-MAKE_FUNC (add_float64, gdouble, gdouble, -1.0, 1.0)
-MAKE_FUNC (add_float32, gfloat, gfloat, -1.0, 1.0)
+MAKE_FUNC_NC (add_float64, gdouble, gdouble)
+MAKE_FUNC_NC (add_float32, gfloat, gfloat)
 /* *INDENT-ON* */
 
+/* we can only accept caps that we and downstream can handle. */
+static GstCaps *
+gst_adder_sink_getcaps (GstPad * pad)
+{
+  GstAdder *adder;
+
+  GstCaps *result, *peercaps, *sinkcaps;
+
+  adder = GST_ADDER (GST_PAD_PARENT (pad));
+
+  GST_OBJECT_LOCK (adder);
+  /* get the downstream possible caps */
+  peercaps = gst_pad_peer_get_caps (adder->srcpad);
+  /* get the allowed caps on this sinkpad, we use the fixed caps function so
+   * that it does not call recursively in this function. */
+  sinkcaps = gst_pad_get_fixed_caps_func (pad);
+  if (peercaps) {
+    /* if the peer has caps, intersect */
+    GST_DEBUG_OBJECT (adder, "intersecting peer and template caps");
+    result = gst_caps_intersect (peercaps, sinkcaps);
+    gst_caps_unref (peercaps);
+    gst_caps_unref (sinkcaps);
+  } else {
+    /* the peer has no caps (or there is no peer), just use the allowed caps
+     * of this sinkpad. */
+    GST_DEBUG_OBJECT (adder, "no peer caps, using sinkcaps");
+    result = sinkcaps;
+  }
+  GST_OBJECT_UNLOCK (adder);
+
+  return result;
+}
+
+/* the first caps we receive on any of the sinkpads will define the caps for all
+ * the other sinkpads because we can only mix streams with the same caps.
+ * */
 static gboolean
 gst_adder_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstAdder *adder;
+
   GList *pads;
+
   GstStructure *structure;
+
   const char *media_type;
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
 
+  GST_LOG_OBJECT (adder, "setting caps on pad %p,%s to %" GST_PTR_FORMAT, pad,
+      GST_PAD_NAME (pad), caps);
+
   /* FIXME, see if the other pads can accept the format. Also lock the
    * format on the other pads to this new format. */
   GST_OBJECT_LOCK (adder);
@@ -206,6 +288,10 @@ gst_adder_setcaps (GstPad * pad, GstCaps * caps)
     GST_DEBUG_OBJECT (adder, "parse_caps sets adder to format float");
     adder->format = GST_ADDER_FORMAT_FLOAT;
     gst_structure_get_int (structure, "width", &adder->width);
+    gst_structure_get_int (structure, "endianness", &adder->endianness);
+
+    if (adder->endianness != G_BYTE_ORDER)
+      goto not_supported;
 
     switch (adder->width) {
       case 32:
@@ -228,6 +314,7 @@ gst_adder_setcaps (GstPad * pad, GstCaps * caps)
 
   return TRUE;
 
+  /* ERRORS */
 not_supported:
   {
     GST_DEBUG_OBJECT (adder, "unsupported format set as caps");
@@ -236,56 +323,185 @@ not_supported:
 }
 
 /* FIXME, the duration query should reflect how long you will produce
- * data, that is the amount of stream time until you will emit EOS. 
- * For synchronized mixing this
- * is always the max of all the durations of upstream since we emit
- * EOS when all of them finished.
+ * data, that is the amount of stream time until you will emit EOS.
+ *
+ * For synchronized mixing this is always the max of all the durations
+ * of upstream since we emit EOS when all of them finished.
+ *
  * We don't do synchronized mixing so this really depends on where the
  * streams where punched in and what their relative offsets are against
  * eachother which we can get from the first timestamps we see.
+ *
  * When we add a new stream (or remove a stream) the duration might
  * also become invalid again and we need to post a new DURATION
- * message to ntify this fact to the parent.
+ * message to notify this fact to the parent.
  * For now we take the max of all the upstream elements so the simple
- * cases work at least somewhat. */
+ * cases work at least somewhat.
+ */
 static gboolean
 gst_adder_query_duration (GstAdder * adder, GstQuery * query)
 {
-  GList *pads;
   gint64 max;
+
   gboolean res;
+
   GstFormat format;
 
-  max = -1;
-  res = TRUE;
+  GstIterator *it;
+
+  gboolean done;
 
   /* parse format */
   gst_query_parse_duration (query, &format, NULL);
 
-  GST_OBJECT_LOCK (adder);
-  pads = GST_ELEMENT_CAST (adder)->sinkpads;
-  for (; pads; pads = g_list_next (pads)) {
-    GstPad *pad = GST_PAD_CAST (pads->data);
-    gint64 duration;
+  max = -1;
+  res = TRUE;
+  done = FALSE;
 
-    /* ask sink peer for duration */
-    res &= gst_pad_query_peer_duration (pad, &format, &duration);
-    /* take max from all valid return values */
-    if (res) {
-      /* valid unknown length, stop searching */
-      if (duration == -1) {
-        max = duration;
+  it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (adder));
+  while (!done) {
+    GstIteratorResult ires;
+
+    gpointer item;
+
+    ires = gst_iterator_next (it, &item);
+    switch (ires) {
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_OK:
+      {
+        GstPad *pad = GST_PAD_CAST (item);
+
+        gint64 duration;
+
+        /* ask sink peer for duration */
+        res &= gst_pad_query_peer_duration (pad, &format, &duration);
+        /* take max from all valid return values */
+        if (res) {
+          /* valid unknown length, stop searching */
+          if (duration == -1) {
+            max = duration;
+            done = TRUE;
+          }
+          /* else see if bigger than current max */
+          else if (duration > max)
+            max = duration;
+        }
+        gst_object_unref (pad);
         break;
       }
-      /* else see if bigger than current max */
-      else if (duration > max)
-        max = duration;
+      case GST_ITERATOR_RESYNC:
+        max = -1;
+        res = TRUE;
+        gst_iterator_resync (it);
+        break;
+      default:
+        res = FALSE;
+        done = TRUE;
+        break;
     }
   }
-  GST_OBJECT_UNLOCK (adder);
+  gst_iterator_free (it);
 
-  /* and store the max */
-  gst_query_set_duration (query, format, max);
+  if (res) {
+    /* and store the max */
+    GST_DEBUG_OBJECT (adder, "Total duration in format %s: %"
+        GST_TIME_FORMAT, gst_format_get_name (format), GST_TIME_ARGS (max));
+    gst_query_set_duration (query, format, max);
+  }
+
+  return res;
+}
+
+static gboolean
+gst_adder_query_latency (GstAdder * adder, GstQuery * query)
+{
+  GstClockTime min, max;
+
+  gboolean live;
+
+  gboolean res;
+
+  GstIterator *it;
+
+  gboolean done;
+
+  res = TRUE;
+  done = FALSE;
+
+  live = FALSE;
+  min = 0;
+  max = GST_CLOCK_TIME_NONE;
+
+  /* Take maximum of all latency values */
+  it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (adder));
+  while (!done) {
+    GstIteratorResult ires;
+
+    gpointer item;
+
+    ires = gst_iterator_next (it, &item);
+    switch (ires) {
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_OK:
+      {
+        GstPad *pad = GST_PAD_CAST (item);
+
+        GstQuery *peerquery;
+
+        GstClockTime min_cur, max_cur;
+
+        gboolean live_cur;
+
+        peerquery = gst_query_new_latency ();
+
+        /* Ask peer for latency */
+        res &= gst_pad_peer_query (pad, peerquery);
+
+        /* take max from all valid return values */
+        if (res) {
+          gst_query_parse_latency (peerquery, &live_cur, &min_cur, &max_cur);
+
+          if (min_cur > min)
+            min = min_cur;
+
+          if (max_cur != GST_CLOCK_TIME_NONE &&
+              ((max != GST_CLOCK_TIME_NONE && max_cur > max) ||
+                  (max == GST_CLOCK_TIME_NONE)))
+            max = max_cur;
+
+          live = live || live_cur;
+        }
+
+        gst_query_unref (peerquery);
+        gst_object_unref (pad);
+        break;
+      }
+      case GST_ITERATOR_RESYNC:
+        live = FALSE;
+        min = 0;
+        max = GST_CLOCK_TIME_NONE;
+        res = TRUE;
+        gst_iterator_resync (it);
+        break;
+      default:
+        res = FALSE;
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (it);
+
+  if (res) {
+    /* store the results */
+    GST_DEBUG_OBJECT (adder, "Calculated total latency: live %s, min %"
+        GST_TIME_FORMAT ", max %" GST_TIME_FORMAT,
+        (live ? "yes" : "no"), GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+    gst_query_set_latency (query, live, min, max);
+  }
 
   return res;
 }
@@ -294,6 +510,7 @@ static gboolean
 gst_adder_query (GstPad * pad, GstQuery * query)
 {
   GstAdder *adder = GST_ADDER (gst_pad_get_parent (pad));
+
   gboolean res = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -321,6 +538,9 @@ gst_adder_query (GstPad * pad, GstQuery * query)
     case GST_QUERY_DURATION:
       res = gst_adder_query_duration (adder, query);
       break;
+    case GST_QUERY_LATENCY:
+      res = gst_adder_query_latency (adder, query);
+      break;
     default:
       /* FIXME, needs a custom query handler because we have multiple
        * sinkpads */
@@ -332,7 +552,24 @@ gst_adder_query (GstPad * pad, GstQuery * query)
   return res;
 }
 
-/* forwards the event to all sinkpads, takes ownership of the 
+static gboolean
+forward_event_func (GstPad * pad, GValue * ret, GstEvent * event)
+{
+  gst_event_ref (event);
+  GST_LOG_OBJECT (pad, "About to send event %s", GST_EVENT_TYPE_NAME (event));
+  if (!gst_pad_push_event (pad, event)) {
+    g_value_set_boolean (ret, FALSE);
+    GST_WARNING_OBJECT (pad, "Sending event  %p (%s) failed.",
+        event, GST_EVENT_TYPE_NAME (event));
+  } else {
+    GST_LOG_OBJECT (pad, "Sent event  %p (%s).",
+        event, GST_EVENT_TYPE_NAME (event));
+  }
+  gst_object_unref (pad);
+  return TRUE;
+}
+
+/* forwards the event to all sinkpads, takes ownership of the
  * event
  *
  * Returns: TRUE if the event could be forwarded on all
@@ -342,34 +579,25 @@ static gboolean
 forward_event (GstAdder * adder, GstEvent * event)
 {
   gboolean ret;
-  GList *pads;
+
+  GstIterator *it;
+  GValue vret = { 0 };
 
   GST_LOG_OBJECT (adder, "Forwarding event %p (%s)", event,
       GST_EVENT_TYPE_NAME (event));
 
   ret = TRUE;
 
-  GST_OBJECT_LOCK (adder);
-  pads = GST_ELEMENT_CAST (adder)->sinkpads;
-  for (; pads; pads = g_list_next (pads)) {
-    GstPad *pad = GST_PAD_CAST (pads->data);
-
-    gst_event_ref (event);
-    ret &= gst_pad_push_event (pad, event);
-
-    if (!ret) {
-      GST_WARNING_OBJECT (pad, "Sending event  %p (%s) failed.",
-          event, GST_EVENT_TYPE_NAME (event));
-      break;
-    } else {
-      GST_LOG_OBJECT (pad, "Sent event  %p (%s).",
-          event, GST_EVENT_TYPE_NAME (event));
-    }
-  }
-  GST_OBJECT_UNLOCK (adder);
-
+  g_value_init (&vret, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&vret, TRUE);
+  it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (adder));
+  gst_iterator_fold (it, (GstIteratorFoldFunction) forward_event_func, &vret,
+      event);
+  gst_iterator_free (it);
   gst_event_unref (event);
 
+  ret = g_value_get_boolean (&vret);
+
   return ret;
 }
 
@@ -377,6 +605,7 @@ static gboolean
 gst_adder_src_event (GstPad * pad, GstEvent * event)
 {
   GstAdder *adder;
+
   gboolean result;
 
   adder = GST_ADDER (gst_pad_get_parent (pad));
@@ -387,9 +616,40 @@ gst_adder_src_event (GstPad * pad, GstEvent * event)
       result = FALSE;
       break;
     case GST_EVENT_SEEK:
-      /* FIXME seek needs something smarter. */
+    {
+      GstSeekFlags flags;
+
+      GstSeekType curtype;
+
+      gint64 cur;
+
+      /* parse the seek parameters */
+      gst_event_parse_seek (event, &adder->segment_rate, NULL, &flags, &curtype,
+          &cur, NULL, NULL);
+
+      /* check if we are flushing */
+      if (flags & GST_SEEK_FLAG_FLUSH) {
+        /* make sure we accept nothing anymore and return WRONG_STATE */
+        gst_collect_pads_set_flushing (adder->collect, TRUE);
+
+        /* flushing seek, start flush downstream, the flush will be done
+         * when all pads received a FLUSH_STOP. */
+        gst_pad_push_event (adder->srcpad, gst_event_new_flush_start ());
+      }
+
+      /* now wait for the collected to be finished and mark a new
+       * segment */
+      GST_OBJECT_LOCK (adder->collect);
+      if (curtype == GST_SEEK_TYPE_SET)
+        adder->segment_position = cur;
+      else
+        adder->segment_position = 0;
+      adder->segment_pending = TRUE;
+      GST_OBJECT_UNLOCK (adder->collect);
+
       result = forward_event (adder, event);
       break;
+    }
     case GST_EVENT_NAVIGATION:
       /* navigation is rather pointless. */
       result = FALSE;
@@ -408,6 +668,7 @@ static gboolean
 gst_adder_sink_event (GstPad * pad, GstEvent * event)
 {
   GstAdder *adder;
+
   gboolean ret;
 
   adder = GST_ADDER (gst_pad_get_parent (pad));
@@ -418,7 +679,7 @@ gst_adder_sink_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_STOP:
       /* mark a pending new segment. This event is synchronized
-       * with the streaming thread so we can safely update the 
+       * with the streaming thread so we can safely update the
        * variable without races. It's somewhat weird because we
        * assume the collectpads forwarded the FLUSH_STOP past us
        * and downstream (using our source pad, the bastard!).
@@ -440,6 +701,7 @@ static void
 gst_adder_class_init (GstAdderClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -452,11 +714,14 @@ gst_adder_class_init (GstAdderClass * klass)
       gst_static_pad_template_get (&gst_adder_src_template));
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gst_adder_sink_template));
-  gst_element_class_set_details (gstelement_class, &adder_details);
+  gst_element_class_set_details_simple (gstelement_class, "Adder",
+      "Generic/Audio",
+      "Add N audio channels together", "Thomas <thomas@apestaart.org>");
 
   parent_class = g_type_class_peek_parent (klass);
 
   gstelement_class->request_new_pad = gst_adder_request_new_pad;
+  gstelement_class->release_pad = gst_adder_release_pad;
   gstelement_class->change_state = gst_adder_change_state;
 }
 
@@ -479,7 +744,7 @@ gst_adder_init (GstAdder * adder)
   gst_element_add_pad (GST_ELEMENT (adder), adder->srcpad);
 
   adder->format = GST_ADDER_FORMAT_UNSET;
-  adder->numpads = 0;
+  adder->padcount = 0;
   adder->func = NULL;
 
   /* keep track of the sinkpads requested */
@@ -504,21 +769,28 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
     const gchar * unused)
 {
   gchar *name;
+
   GstAdder *adder;
+
   GstPad *newpad;
 
-  g_return_val_if_fail (GST_IS_ADDER (element), NULL);
+  gint padcount;
 
   if (templ->direction != GST_PAD_SINK)
     goto not_sink;
 
   adder = GST_ADDER (element);
 
-  name = g_strdup_printf ("sink%d", adder->numpads);
+  /* increment pad counter */
+  padcount = g_atomic_int_exchange_and_add (&adder->padcount, 1);
+
+  name = g_strdup_printf ("sink%d", padcount);
   newpad = gst_pad_new_from_template (templ, name);
+  GST_DEBUG_OBJECT (adder, "request new pad %s", name);
+  g_free (name);
 
   gst_pad_set_getcaps_function (newpad,
-      GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
+      GST_DEBUG_FUNCPTR (gst_adder_sink_getcaps));
   gst_pad_set_setcaps_function (newpad, GST_DEBUG_FUNCPTR (gst_adder_setcaps));
   gst_collect_pads_add_pad (adder->collect, newpad, sizeof (GstCollectData));
 
@@ -528,11 +800,10 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
   adder->collect_event = (GstPadEventFunction) GST_PAD_EVENTFUNC (newpad);
   gst_pad_set_event_function (newpad, GST_DEBUG_FUNCPTR (gst_adder_sink_event));
 
+  /* takes ownership of the pad */
   if (!gst_element_add_pad (GST_ELEMENT (adder), newpad))
     goto could_not_add;
 
-  adder->numpads++;
-
   return newpad;
 
   /* errors */
@@ -543,12 +814,26 @@ not_sink:
   }
 could_not_add:
   {
+    GST_DEBUG_OBJECT (adder, "could not add pad");
     gst_collect_pads_remove_pad (adder->collect, newpad);
     gst_object_unref (newpad);
     return NULL;
   }
 }
 
+static void
+gst_adder_release_pad (GstElement * element, GstPad * pad)
+{
+  GstAdder *adder;
+
+  adder = GST_ADDER (element);
+
+  GST_DEBUG_OBJECT (adder, "release pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+
+  gst_collect_pads_remove_pad (adder->collect, pad);
+  gst_element_remove_pad (element, pad);
+}
+
 static GstFlowReturn
 gst_adder_collected (GstCollectPads * pads, gpointer user_data)
 {
@@ -563,45 +848,60 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
    * - push out the output buffer
    */
   GstAdder *adder;
+
   guint size;
+
   GSList *collected;
+
   GstBuffer *outbuf;
+
   GstFlowReturn ret;
+
   gpointer outbytes;
 
+  gboolean empty = TRUE;
+
   adder = GST_ADDER (user_data);
 
   /* this is fatal */
   if (G_UNLIKELY (adder->func == NULL))
     goto not_negotiated;
 
-  outbuf = NULL;
-  outbytes = NULL;
-
   /* get available bytes for reading, this can be 0 which could mean
    * empty buffers or EOS, which we will catch when we loop over the
    * pads. */
   size = gst_collect_pads_available (pads);
 
   GST_LOG_OBJECT (adder,
-      "starting to cycle through channels, %d bytes available", size);
+      "starting to cycle through channels, %d bytes available (bps = %d)", size,
+      adder->bps);
+
+  outbuf = NULL;
+  outbytes = NULL;
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
     GstCollectData *data;
+
     guint8 *bytes;
+
     guint len;
 
+    GstBuffer *inbuf;
+
     data = (GstCollectData *) collected->data;
 
-    /* get pointer to copy size bytes */
-    len = gst_collect_pads_read (pads, data, &bytes, size);
-    /* length 0 means EOS or an empty buffer so we still need to flush in
+    /* get a subbuffer of size bytes */
+    inbuf = gst_collect_pads_take_buffer (pads, data, size);
+    /* NULL means EOS or an empty buffer so we still need to flush in
      * case of an empty buffer. */
-    if (len == 0) {
+    if (inbuf == NULL) {
       GST_LOG_OBJECT (adder, "channel %p: no bytes available", data);
       goto next;
     }
 
+    bytes = GST_BUFFER_DATA (inbuf);
+    len = GST_BUFFER_SIZE (inbuf);
+
     if (outbuf == NULL) {
       GST_LOG_OBJECT (adder, "channel %p: making output buffer of %d bytes",
           data, size);
@@ -612,36 +912,51 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
       outbytes = GST_BUFFER_DATA (outbuf);
       gst_buffer_set_caps (outbuf, GST_PAD_CAPS (adder->srcpad));
 
-      /* clear if we are only going to fill a partial buffer */
-      if (G_UNLIKELY (size > len))
+      if (!GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
+        /* clear if we are only going to fill a partial buffer */
+        if (G_UNLIKELY (size > len))
+          memset (outbytes, 0, size);
+
+        GST_LOG_OBJECT (adder, "channel %p: copying %d bytes from data %p",
+            data, len, bytes);
+
+        /* and copy the data into it */
+        memcpy (outbytes, bytes, len);
+        empty = FALSE;
+      } else {
+        GST_LOG_OBJECT (adder, "channel %p: zeroing %d bytes from data %p",
+            data, len, bytes);
         memset (outbytes, 0, size);
-
-      GST_LOG_OBJECT (adder, "channel %p: copying %d bytes from data %p",
-          data, len, bytes);
-
-      /* and copy the data into it */
-      memcpy (outbytes, bytes, len);
+      }
     } else {
-      GST_LOG_OBJECT (adder, "channel %p: mixing %d bytes from data %p",
-          data, len, bytes);
-      /* other buffers, need to add them */
-      adder->func ((gpointer) outbytes, (gpointer) bytes, len);
+      if (!GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
+        GST_LOG_OBJECT (adder, "channel %p: mixing %d bytes from data %p",
+            data, len, bytes);
+        /* other buffers, need to add them */
+        adder->func ((gpointer) outbytes, (gpointer) bytes, len);
+        empty = FALSE;
+      } else {
+        GST_LOG_OBJECT (adder, "channel %p: skipping %d bytes from data %p",
+            data, len, bytes);
+      }
     }
   next:
-    gst_collect_pads_flush (pads, data, len);
+    if (inbuf)
+      gst_buffer_unref (inbuf);
   }
 
   /* can only happen when no pads to collect or all EOS */
   if (outbuf == NULL)
     goto eos;
 
-  /* our timestamping is very simple, just an ever incrementing 
+  /* our timestamping is very simple, just an ever incrementing
    * counter, the new segment time will take care of their respective
    * stream time. */
   if (adder->segment_pending) {
     GstEvent *event;
 
     /* FIXME, use rate/applied_rate as set on all sinkpads.
+     * - currently we just set rate as received from last seek-event
      * We could potentially figure out the duration as well using
      * the current segment positions and the stated stop positions.
      * Also we just start from stream time 0 which is rather
@@ -653,11 +968,12 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
      * the later streams would be delayed until the stream times
      * match.
      */
-    event = gst_event_new_new_segment_full (FALSE, 1.0,
-        1.0, GST_FORMAT_TIME, adder->timestamp, -1, 0);
+    event = gst_event_new_new_segment_full (FALSE, adder->segment_rate,
+        1.0, GST_FORMAT_TIME, adder->timestamp, -1, adder->segment_position);
 
     gst_pad_push_event (adder->srcpad, event);
     adder->segment_pending = FALSE;
+    adder->segment_position = 0;
   }
 
   /* set timestamps on the output buffer */
@@ -674,6 +990,10 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   GST_BUFFER_DURATION (outbuf) = adder->timestamp -
       GST_BUFFER_TIMESTAMP (outbuf);
 
+  /* if we only processed silence, mark output again as silence */
+  if (empty)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
+
   /* send it out */
   GST_LOG_OBJECT (adder, "pushing outbuf, timestamp %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
@@ -700,6 +1020,7 @@ static GstStateChangeReturn
 gst_adder_change_state (GstElement * element, GstStateChange transition)
 {
   GstAdder *adder;
+
   GstStateChangeReturn ret;
 
   adder = GST_ADDER (element);
@@ -711,6 +1032,8 @@ gst_adder_change_state (GstElement * element, GstStateChange transition)
       adder->timestamp = 0;
       adder->offset = 0;
       adder->segment_pending = TRUE;
+      adder->segment_position = 0;
+      adder->segment_rate = 1.0;
       gst_segment_init (&adder->segment, GST_FORMAT_UNDEFINED);
       gst_collect_pads_start (adder->collect);
       break;
