@@ -21,14 +21,18 @@
  * SECTION:element-gdpdepay
  * @see_also: gdppay
  *
+ * <refsect2>
+ * <para>
  * This element depayloads GStreamer Data Protocol buffers back to deserialized
  * buffers and events.
- *
- * <refsect2>
- * |[
+ * </para>
+ * <para>
+ * <programlisting>
  * gst-launch -v -m filesrc location=test.gdp ! gdpdepay ! xvimagesink
- * ]| This pipeline plays back a serialized video stream as created in the
+ * </programlisting>
+ * This pipeline plays back a serialized video stream as created in the
  * example for gdppay.
+ * </para>
  * </refsect2>
  */
 
@@ -110,7 +114,7 @@ gst_gdp_depay_class_init (GstGDPDepayClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_gdp_depay_change_state);
-  gobject_class->finalize = gst_gdp_depay_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_gdp_depay_finalize);
 }
 
 static void
@@ -160,13 +164,8 @@ gst_gdp_depay_sink_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
-      /* forward flush start */
-      res = gst_pad_push_event (this->srcpad, event);
-      break;
     case GST_EVENT_FLUSH_STOP:
-      /* clear adapter on flush */
-      gst_adapter_clear (this->adapter);
-      /* forward flush stop */
+      /* forward flush start and stop */
       res = gst_pad_push_event (this->srcpad, event);
       break;
     case GST_EVENT_EOS:
@@ -463,7 +462,6 @@ gst_gdp_depay_change_state (GstElement * element, GstStateChange transition)
         gst_caps_unref (this->caps);
         this->caps = NULL;
       }
-      gst_adapter_clear (this->adapter);
       break;
     default:
       break;
