@@ -16,7 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+ 
 #ifndef __GST_PLAY_SINK_H__
 #define __GST_PLAY_SINK_H__
 
@@ -36,8 +36,6 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PLAY_SINK))
 #define GST_IS_PLAY_SINK_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PLAY_SINK))
-#define GST_PLAY_SINK_CAST(obj) \
-  ((GstPlaySink*)(obj))
 
 /**
  * GstPlaySinkType:
@@ -47,7 +45,6 @@ G_BEGIN_DECLS
  * @GST_PLAY_SINK_TYPE_VIDEO_RAW: a raw video pad
  * @GST_PLAY_SINK_TYPE_TEXT: a raw text pad
  * @GST_PLAY_SINK_TYPE_LAST: the last type
- * @GST_PLAY_SINK_TYPE_FLUSHING: a flushing pad, used when shutting down
  *
  * Types of pads that can be requested from the sinks.
  */
@@ -57,10 +54,7 @@ typedef enum {
   GST_PLAY_SINK_TYPE_VIDEO     = 2,
   GST_PLAY_SINK_TYPE_VIDEO_RAW = 3,
   GST_PLAY_SINK_TYPE_TEXT      = 4,
-  GST_PLAY_SINK_TYPE_LAST      = 5,
-
-  /* this is a dummy pad */
-  GST_PLAY_SINK_TYPE_FLUSHING  = 6
+  GST_PLAY_SINK_TYPE_LAST      = 5
 } GstPlaySinkType;
 
 typedef struct _GstPlaySink GstPlaySink;
@@ -71,8 +65,11 @@ GType gst_play_sink_get_type (void);
 GstPad *         gst_play_sink_request_pad    (GstPlaySink *playsink, GstPlaySinkType type);
 void             gst_play_sink_release_pad    (GstPlaySink *playsink, GstPad *pad);
 
-void             gst_play_sink_set_sink       (GstPlaySink * playsink, GstPlaySinkType type, GstElement * sink);
-GstElement *     gst_play_sink_get_sink       (GstPlaySink * playsink, GstPlaySinkType type);
+void             gst_play_sink_set_video_sink (GstPlaySink * playsink, GstElement * sink);
+GstElement *     gst_play_sink_get_video_sink (GstPlaySink * playsink);
+
+void             gst_play_sink_set_audio_sink (GstPlaySink * playsink, GstElement * sink);
+GstElement *     gst_play_sink_get_audio_sink (GstPlaySink * playsink);
 
 void             gst_play_sink_set_vis_plugin (GstPlaySink * playsink, GstElement * vis);
 GstElement *     gst_play_sink_get_vis_plugin (GstPlaySink * playsink);
@@ -88,15 +85,11 @@ GstPlayFlags     gst_play_sink_get_flags      (GstPlaySink * playsink);
 
 void             gst_play_sink_set_font_desc  (GstPlaySink *playsink, const gchar * desc);
 gchar *          gst_play_sink_get_font_desc  (GstPlaySink *playsink);
-void             gst_play_sink_set_subtitle_encoding  (GstPlaySink *playsink, const gchar * encoding);
-gchar *          gst_play_sink_get_subtitle_encoding  (GstPlaySink *playsink);
 
 GstBuffer *      gst_play_sink_get_last_frame (GstPlaySink * playsink);
 
 gboolean         gst_play_sink_reconfigure    (GstPlaySink * playsink);
 
-gboolean         gst_play_sink_plugin_init    (GstPlugin * plugin);
-
 G_END_DECLS
 
 #endif /* __GST_PLAY_SINK_H__ */
